<?xml version="1.0" encoding="iso-8859-1"?>
<!--
    ResultXSLT(tm) - an environment for generating stylesheet fragments

    $Id: result.xsl,v 1.12 2004/05/24 23:24:57 G. Ken Holman Exp $
-->
<!--
Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-rxslts.htm

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->
<!DOCTYPE xst:stylesheet
[
  <!ENTITY resultns "http://www.CraneSoftwrights.com/ns/result">
  <!ENTITY resultattrns "http://www.CraneSoftwrights.com/ns/result/attr">
  <!ENTITY resultattrpns "http://www.CraneSoftwrights.com/ns/result/attrp">
]>
<xst:stylesheet xmlns:xst="http://www.w3.org/1999/XSL/Transform"
                xmlns:x="&resultns;"
                xmlns:xa="&resultattrns;"
                xmlns:xap="&resultattrpns;"
                xmlns:xsl="dummy"
                exclude-result-prefixes="x xa xap xsl"
                version="1.0">

<!--this cleans up the output when using indent="yes"; acceptable for use
    with XSL-FO but this may not be acceptable for other vocabularies-->
<xst:strip-space elements="*"/>
<xst:output indent="yes"/>

<!--a stylesheet writing a stylesheet-->
<xst:namespace-alias stylesheet-prefix="xsl" result-prefix="xst"/>

<!--distinguishing paths out from and in to the result stylesheet fragment-->
<xst:variable name="out-prefix" select="concat(/*/@x:out-prefix,':')"/>
<xst:variable name="in-prefix" select="concat(/*/@x:in-prefix,':')"/>

<!--branches identified for the result stylesheet fragment-->
<xst:key name="branches" match="*[@x:branch]" use="@x:branch"/>
<xst:key name="branches" match="*[@x:branch-children]" 
         use="@x:branch-children"/>

<!--build the stylesheet fragment-->
<xst:template match="/">
  <!--leading comments-->
  <xst:apply-templates select="/comment()[following-sibling::*]"/>

  <!--stylesheet body-->
  <xst:text>
</xst:text>
  <xsl:stylesheet>
    <xst:copy-of select="/*/namespace::*[.!='&resultns;' and
                                         .!='&resultattrpns;' and
                                         .!='&resultattrns;']"/>
    <xst:attribute name="version">1.0</xst:attribute>
    <xst:attribute name="exclude-result-prefixes">
      <xst:for-each select="/*/namespace::*[name(.)=
                                          (/*/@x:in-prefix|/*/@x:out-prefix)]">
        <xst:if test="position()>1" xml:space="preserve"> </xst:if>
        <xst:value-of select="name(.)"/>
      </xst:for-each>
    </xst:attribute>
    <xst:text>
</xst:text>

    <!--collect all constructs that are going to be top-level elements-->
    <xst:variable name="all"
      select=" / | //x:*
     | //*[@x:branch]
                 [generate-id(.)=generate-id(key('branches',@x:branch)[1])]
     | //*[@x:branch-children]
         [generate-id(.)=generate-id(key('branches',@x:branch-children)[1])]"/>

    <!--this is split up to accommodate absent sort attributes as value zero-->
    <xst:apply-templates select="$all[@x:sort&lt;0]" mode="construct">
      <xst:sort select="@x:sort" data-type="number"/>
    </xst:apply-templates>
    <xst:apply-templates select="$all[not(@x:sort) or @x:sort=0]" 
                         mode="construct">
    </xst:apply-templates>
    <xst:apply-templates select="$all[@x:sort&gt;0]" mode="construct">
      <xst:sort select="@x:sort" data-type="number"/>
    </xst:apply-templates>

    <!--finished the stylesheet-->
  </xsl:stylesheet>
  <!--trailing comments-->
  <xst:apply-templates select="/comment()[preceding-sibling::*]"/>
</xst:template>

<!--========================================================================-->
<!--top-level elements of the result-->

<!--begin the output stylesheet-->
<xst:template mode="construct" match="/">
  <xst:text>

</xst:text>
<xst:comment>Root: default processing for the root node</xst:comment>
  <xsl:template match="/">
    <xst:apply-templates select="*"/>
  </xsl:template>
</xst:template>

<!--declare top-level variables-->
<xst:template mode="construct" match="x:decl">
  <xst:apply-templates select="@x:comment"/>
  <xsl:variable name="{@name}" select="{@value}"/>
</xst:template>

<!--create branch out/in pairs-->
<xst:template mode="construct" match="*[@x:branch | @x:branch-children]">
  <xst:if test="@x:branch">
    <!--current or children of current need to be replaced with a call-->
    <xst:apply-templates select="@x:comment"/>
    <xst:text>

</xst:text>
    <xst:comment>
      <xst:text/>Branch: <xst:value-of select="@x:branch"/>
    </xst:comment>
    <xsl:template name="{$out-prefix}{@x:branch}">
      <xst:choose>
        <xst:when test="@x:branch">
          <xsl:call-template name="{$in-prefix}{@x:branch}"/>
        </xst:when>
        <xst:otherwise>
          <xst:apply-templates select="."/>
        </xst:otherwise>
      </xst:choose>
    </xsl:template>
    <xst:if test="@x:branch">
      <xsl:template name="{$in-prefix}{@x:branch}">
        <xst:choose>
          <xst:when test="@x:branch-children">
            <xst:call-template name="copy-with-branch-children"/>
          </xst:when>
          <xst:otherwise>
            <xst:call-template name="copy"/>
          </xst:otherwise>
        </xst:choose>
      </xsl:template>
    </xst:if>
  </xst:if>
  <xst:if test="( @x:branch-children and 
          generate-id(.)=generate-id(key('branches',@x:branch-children)[1]) )">
    <xst:apply-templates select="@x:comment-children"/>
    <xst:text>

</xst:text>
    <xst:comment>
      <xst:text/>Branch: <xst:value-of select="@x:branch-children"/>
    </xst:comment>
    <xsl:template name="{$out-prefix}{@x:branch-children}">
      <xsl:call-template name="{$in-prefix}{@x:branch-children}"/>
    </xsl:template>
    <xst:if test="@x:branch-children">
      <xsl:template name="{$in-prefix}{@x:branch-children}">
        <xst:apply-templates select="node()"/>
      </xsl:template>
    </xst:if>
  </xst:if>
</xst:template>

<!--don't bother with ignored branches; provided a branch does indeed exist-->
<xst:template match="*[@x:ignore]" priority="1">
  <xst:if test="not(key('branches',@x:ignore))">
    <xst:message>
      <xst:text>Unsatisfied ignore of branch "</xst:text>
      <xst:value-of select="@x:ignore"/>"<xst:text/>
    </xst:message>
  </xst:if>
</xst:template>

<!--branch out the given element-->
<xst:template match="*[@x:branch]" priority="1">
  <xsl:call-template name="{$out-prefix}{@x:branch}"/>
</xst:template>

<!--branch out the given element's children-->
<xst:template match="*[@x:branch-children]" name="copy-with-branch-children">
  <xst:element name="{name(.)}" namespace="{namespace-uri(.)}">
    <xst:apply-templates select="@*[namespace-uri(.)!='&resultattrns;']"/>
    <xst:apply-templates select="@xa:*"/><!--namespace-uri(.)=&resultattrns;-->
    <xsl:call-template name="{$out-prefix}{@x:branch-children}"/>
  </xst:element>
</xst:template>

<!--no branching going on for this element-->
<xst:template match="*" name="copy"><!--typical process-->
  <!--
  <xst:message><xst:call-template name="xpath"/></xst:message>
  -->
  <xst:element name="{name(.)}" namespace="{namespace-uri(.)}">
    <xst:apply-templates select="@*[namespace-uri(.)!='&resultattrns;']"/>
    <xst:apply-templates select="@xa:*"/><!--namespace-uri(.)=&resultattrns;-->
    <xst:apply-templates select="node()"/>
  </xst:element>
</xst:template>

<!--leaf nodes are just copied-->
<xst:template match="@*|text()|processing-instruction()">
  <xst:copy/>
</xst:template>

<!--overrides do not accommodate all namespace-qualified attributes-->
<xst:template match="@xa:*"><!--override any existing attrs with given-->
  <!--
  <xst:message><xst:value-of select="concat(name(.),'=',.)"/></xst:message>
  -->
  <xst:variable name="prefix">
    <xst:for-each select="@xap:*[local-name(.)=local-name(current())]">
      <xst:value-of select="."/>:<xst:text/>
    </xst:for-each>
  </xst:variable>
  <xst:attribute name="{$prefix}{local-name(.)}">
    <xst:value-of select="."/>
  </xst:attribute>
</xst:template>

<xst:template match="@x:*|x:*"/><!--strip unneeded elements and attributes-->

<!--match comments; start them on a new line (may not be necessary for some
    XSLT processors but for Saxon 6.5.3 this accommodates the apparent loss
    of the text node before comments)-->

<xst:template match="comment()">
  <xst:text>
</xst:text>
  <xst:comment>
    <xst:value-of select="."/>
  </xst:comment>
</xst:template>

<!--generate comments by annotation request-->

<xst:template match="@x:comment | @x:comment-children">
  <xst:text>

</xst:text>
  <xst:comment>
    <xst:value-of select="."/>
  </xst:comment>
</xst:template>

<!--========================================================================-->
<!--debug stuff; only engaged when needed during development-->

<xst:template name="xpath">
  <xst:for-each select="ancestor-or-self::* | .">
    <xst:text>/</xst:text>
    <xst:if test="not(self::*)">@</xst:if>
    <xst:value-of select="name(.)"/>
    <xst:if test="preceding-sibling::*[name(.)=name(current())]">
      <xst:text>[</xst:text>
      <xst:value-of 
            select="count(preceding-sibling::*[name(.)=name(current())]) + 1"/>
      <xst:text>]</xst:text>
    </xst:if>
  </xst:for-each>
</xst:template>

</xst:stylesheet>
